/*
*  Name:      string.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type string
*  Date:      $Date: 2003/10/06 13:21:35 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_STRING__
#define _UESQL_STRING__

#include <mpcl/text/string.hh>
#include "type.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  ///  SQL type string.
  class TString                :
    public IType               ,
    public mpcl::text::TString
  {

    public:

      ///  Import this type.
      typedef
        mpcl::text::TString::size_type
        size_type;


    public:
      
      //
      //  C O N S T R U C T O R S
      //
      
      explicit TString (void)  :
        IType               () ,
        mpcl::text::TString () {}

      ///  Builds a new instance from another instance.
      TString (const TString& rkySTRING) :
        IType               ()           ,
        mpcl::text::TString (rkySTRING)
      {
        setNull (rkySTRING.isNull());
      }

      TString ( const TString& rkySTRING      ,
                size_type      zOFFSET        ,
                size_type      zLENGTH = npos )           :
        IType               ()                            ,
        mpcl::text::TString (rkySTRING, zOFFSET, zLENGTH)
      {
        setNull (rkySTRING.isNull());
      }

      TString (const mpcl::text::TString& rkySOURCE) :
        IType               ()                       ,
        mpcl::text::TString (rkySOURCE)
      {
        setNull (false);
      }

      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @param zLENGTH  Source string length.
      */
      TString (const char* pkcVALUE, size_type zLENGTH) :
        IType               ()                          ,
        mpcl::text::TString ()
      {
        if ( pkcVALUE )
        {
          setNull (false);
          assign (pkcVALUE, zLENGTH);
        }
      }
      
      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      TString (const char* pkcVALUE) :
        IType               ()       ,
        mpcl::text::TString ()
      {
        if ( pkcVALUE )
        {
          setNull (false);
          assign (pkcVALUE);
        }
      }
      
      TString (size_type zTIMES, char cSOURCE) :
        IType               ()                 ,
        mpcl::text::TString (zTIMES, cSOURCE)
      {
        setNull (false);
      }
      
      template<typename InputIterator>
      TString (InputIterator tBEGIN_ITER, InputIterator tEND_ITER) :
        IType               ()                                     ,
        mpcl::text::TString (tBEGIN_ITER, tEND_ITER)
      {
        setNull (false);
      }

      TString& operator = (const TString& rkySTRING)
      {
        mpcl::text::TString::operator = (rkySTRING);
        setNull (rkySTRING.isNull());
        return *this;
      }

      TString& operator = (const mpcl::text::TString& rkySOURCE)
      {
        mpcl::text::TString::operator = (rkySOURCE);
        setNull (false);
        return *this;
      }

      TString& operator = (const char* pkcVALUE)
      {
        if ( !pkcVALUE )
        {
          throw mpcl::TConstraintException ("null string", __FILE__, __LINE__);
        }
        mpcl::text::TString::operator = (pkcVALUE);
        setNull (false);
        return *this;
      }

      TString& operator = (char cSOURCE)
      {
        mpcl::text::TString::operator = (cSOURCE);
        setNull (false);
        return *this;
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null,  then the instance
      *  is null.
      *  @pre ( !pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE)
      {
        if ( !pkcVALUE )
        {
          erase();
          setNull (true);
        }
        else
        {
          assign (pkcVALUE);
          setNull (false);
        }
      }

      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        assign (rkyVALUE);
        setNull (false);
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::text::TString get (void) const
      {

        mpcl::text::TString   yResult;
        mpcl::text::TString   yEscaped (*this);

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yEscaped.replaceAll ("'", "''");
          yEscaped.replaceAll ("\\", "\\\\");
          yResult = '\'';
          yResult.append (yEscaped);
          yResult.append (1, '\'');
        }
        return yResult;

      }  // get()

  };  // class TString

}  // namespace uesqlc


#endif  // not _UESQL_STRING__
