/*
*  Name:      time.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type time
*  Date:      $Date: 2003/10/06 13:21:35 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_TIME__
#define _UESQL_TIME__

#include <mpcl/text/local_date.hh>
#include <mpcl/text/string.hh>
#include "type.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  using mpcl::text::TLocalDate;

  ///  SQL time handler.
  class TTime         :
    public TLocalDate ,
    public IType
  {

    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TTime (void)    :
        TLocalDate () ,
        IType      () {}

      ///  Builds a new instance from another instance.
      TTime (const TTime& rktTIME) :
        TLocalDate (rktTIME)       ,
        IType      ()
      {
        setNull (rktTIME.isNull());
      }

      /**
      *  Builds a new instance.  It builds  the instance with an useless  valid
      *  date to let the base class 'mpcl::text::TLocalDate' be used.
      *  @param kiHOUR   Hours.
      *  @param kiMINUTE Minutes.
      *  @param kiSECOND Seconds.
      *  @see mpcl::text::TLocalDate.
      */
      TTime (const int kiHOUR, const int kiMINUTE, const int kiSECOND) :
        TLocalDate (1970, 1, 1, kiHOUR, kiMINUTE, kiSECOND)            ,
        IType      ()
      {
        setNull (false);
      }

      explicit TTime (const char* pkcTIME) :
        TLocalDate (eEmpty)                ,
        IType      ()
      {
        set (pkcTIME);
      }

      explicit TTime (const std::string& rkyTIME) :
        TLocalDate (eEmpty)                       ,
        IType      ()
      {
        set (rkyTIME);
      }

      /**
      *  Reserves  a  buffer \a kzSIZE  octets  length,  to  let somebody store
      *  a time as a zero-terminated string of chars.
      *  @param kzSIZE Size of the buffer.
      *  @return The buffer.
      */
      void* getBuffer (const std::size_t kzSIZE);

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE);

      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      mpcl::text::TString get (void) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get ("'%T'");
        }
        return yResult;
      }

      /**
      *  Returns time in SQL format.
      *  @param pkcTIME_FORMAT Time format in strftime-like format. It defaults
      *                        to ANSI SQL time  format.  A  valid time  can be
      *                        '23:59:59'.
      *  @return Time in readable format.
      *  @warning pkcTIME_FORMAT must include the enclosing quotes (').
      */
      mpcl::text::TString get (const char* pkcTIME_FORMAT) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get (pkcTIME_FORMAT);
        }
        return yResult;
      }

  };  // class TTime

}  // namespace uesqlc


#endif  // not _UESQL_TIME__
