#include <mpcl/util/logging/log.hh>
#include "odbc-3.x.hh"

#line 1 "../test-000-cursors.uesql.cc"
/*
*  Name:         test-000-cursors.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 1996-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using std::cerr;
  using std::endl;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TString       yVar0;
  TString       yVar1;
  TReal         tHours;
  TInteger      tZero (0L);
  TIdentifier   yAttribName ("work_code");
  bool          gEmpty       = true;
  int           iRecordCount = 0;

  TEST_INIT ("test for cursors");

#line 84 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 85 "../test-000-cursors.uesql.cc"

#line 86 "../test-000-cursors.uesql.cc"

#line 90 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT NAME, HOURS FROM WORKERS WHERE ( NAME = 'nobody' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 91 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();


}  // not ODBC 3.x
#line 92 "../test-000-cursors.uesql.cc"

  while ( true )
  {
#line 96 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 97 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar1);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 98 "../test-000-cursors.uesql.cc"

    gEmpty = false;
  }

loop_end:
#line 104 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 105 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 106 "../test-000-cursors.uesql.cc"

  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                       HOURS\n");
    fprintf (hptTestTargetFile, "========= ========================================== =====\n");
  }
#line 122 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 126 "../test-000-cursors.uesql.cc"

#line 127 "../test-000-cursors.uesql.cc"

#line 132 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yAttribName);

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT $z, NAME, HOURS FROM WORKERS;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 133 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();


#line 134 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (tZero);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetchScroll (hrtResultTriple.tSqlhstmt, SQL_FETCH_ABSOLUTE, std::atoi (uesqlc::htOdbc3xManager.instantiate ("$z").c_str()));
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end_2;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 135 "../test-000-cursors.uesql.cc"

  TEST_FAIL;
  goto next;

loop_end_2:
  TEST_PASS;

next:  
  while ( true )
  {
#line 146 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 147 "../test-000-cursors.uesql.cc"

#line 148 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto loop_end_3;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 149 "../test-000-cursors.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5.1f\n", yVar0.get().c_str(), yVar1.get().c_str(), (double) tHours);
    }
    ++iRecordCount;
  }

loop_end_3:
#line 159 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 163 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 164 "../test-000-cursors.uesql.cc"

  if ( iRecordCount > 0 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  TString*           pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 262 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 266 "../test-000-cursors.uesql.cc"

#line 267 "../test-000-cursors.uesql.cc"

#line 272 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString              hySqlSchemaLocation;
mpcl::text::TString              hyDatabaseName;
mpcl::text::TString              hyConnectionName;
mpcl::text::TString              hyUserLogin;
mpcl::text::TString::size_type   zSolidusPos;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

//
//  '$z' has been omitted.
//
hySqlSchemaLocation = uesqlc::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = "DEFAULT";
hyUserLogin         = uesqlc::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::text::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  uesqlc::htOdbc3xManager.unbindConnection (hyConnectionName);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 281 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("CREATE TABLE JOBS (CODE CHARACTER (3) NOT NULL, WORKER_COUNT INTEGER);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 288 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("CREATE TABLE WORKERS (WORK_CODE CHARACTER (3) NOT NULL, NAME CHARACTER (40) NOT NULL, HOURS DECIMAL (5));");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 293 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("ALTER TABLE WORKERS ADD COLUMN STATE CHARACTER (40);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 294 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INTEGER;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 299 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yCodeValue);

htResultTriple = uesqlc::htOdbc3xManager.execute ("DELETE FROM JOBS WHERE ( CODE = $z );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 300 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DELETE FROM JOBS WHERE ( CODE = '001' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 301 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DELETE FROM WORKERS;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 306 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO JOBS VALUES ('001', 100);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 307 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO JOBS VALUES ('002', 250);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 308 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 309 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 310 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 311 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 312 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 313 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'william', NULL, 'Missouri', 5550005);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 314 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'peter', NULL, 'Arkansas', 5550006);");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 315 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 316 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 321 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tCondition);

uesqlc::htOdbc3xManager.popFromOutputQueue()->set (uesqlc::htOdbc3xManager.instantiate ("( WORK_CODE = CODE )"));
#line 326 "../test-000-cursors.uesql.cc"

#line 333 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar1);
uesqlc::htOdbc3xManager.pushOnInputQueue (tCondition);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT HOURS FROM WORKERS, JOBS WHERE $z;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto cardinality_check;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto cardinality_check;
  }
}
}
}  // not ODBC 3.x
#line 334 "../test-000-cursors.uesql.cc"

    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::cardinality_violation::pkcNO_SUBCLASS);
#line 340 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 344 "../test-000-cursors.uesql.cc"

#line 349 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tCount);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT COUNT (*) FROM WORKERS WHERE ( HOURS IS NULL );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 354 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT WORK_CODE, HOURS FROM WORKERS WHERE ( HOURS IS NULL );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 355 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();


#line 356 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tHours);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto not_found;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
#line 357 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 358 "../test-000-cursors.uesql.cc"

    TEST_NUMBERS (tCount,          4);
    TEST_NUMBERS (tCount.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),  false);
    TEST_NUMBERS (tHours.isNull(), true);
    
    yCursorName = "mycursor";
#line 366 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 370 "../test-000-cursors.uesql.cc"

#line 375 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tCount);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto not_found;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    goto error;
  }
}
}
#line 380 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tCondition);

uesqlc::htOdbc3xManager.popFromOutputQueue()->set (uesqlc::htOdbc3xManager.instantiate ("( WORKERS.NAME = 'myname' )"));
#line 381 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&tCondition);
uesqlc::htOdbc3xManager.pushOnInputQueue (tCondition);

uesqlc::htOdbc3xManager.popFromOutputQueue()->set (uesqlc::htOdbc3xManager.instantiate ("( $z AND ( WORKERS.NAME = NAME ) )"));
#line 388 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (tCondition);

htResultTriple = uesqlc::htOdbc3xManager.execute ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE $z;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 389 "../test-000-cursors.uesql.cc"

    TEST_FUNCTION (_TestCursors());
#line 392 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 396 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  goto error;
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
#line 397 "../test-000-cursors.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();


#line 398 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yCursorName);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&yVar0);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("$z");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  goto not_found;
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
#line 399 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("MYCURSOR");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  goto error;
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 400 "../test-000-cursors.uesql.cc"

    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::successful_completion::pkcNO_SUBCLASS);
    TEST_NUMBERS (tCount,        4);
    TEST_STRINGS (yVar0.c_str(), "001");
#line 405 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 409 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("REVOKE ALL PRIVILEGES ON JOBS FROM PUBLIC CASCADE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 410 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("REVOKE ALL PRIVILEGES ON WORKERS FROM PUBLIC RESTRICT;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 415 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("GRANT SELECT ON JOBS TO PUBLIC;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 416 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("GRANT SELECT ON WORKERS TO PUBLIC;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 417 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("GRANT DELETE ON WORKERS TO PUBLIC;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 422 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 427 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DROP VIEW VIEW1 CASCADE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 432 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DROP TABLE JOBS CASCADE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 437 "../test-000-cursors.uesql.cc"
{
uesqlc::htOdbc3xManager.clearQueues();

if ( uesqlc::htOdbc3xManager.existsDefaultConnection() )
{
  uesqlc::htOdbc3xManager.resetStatus();
  uesqlc::htOdbc3xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08003";
  goto error;
}
}
}  // not ODBC 3.x
#line 438 "../test-000-cursors.uesql.cc"


#line 441 "../test-000-cursors.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 445 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString              hySqlSchemaLocation;
mpcl::text::TString              hyDatabaseName;
mpcl::text::TString              hyConnectionName;
mpcl::text::TString              hyUserLogin;
mpcl::text::TString::size_type   zSolidusPos;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htOdbc3xManager.pushOnInputQueue (*pyDbmsUserLogin);

hySqlSchemaLocation = uesqlc::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = uesqlc::htOdbc3xManager.instantiate ("_'myconnection'");
hyUserLogin         = uesqlc::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
hyConnectionName.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::text::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  uesqlc::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 450 "../test-000-cursors.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;

uesqlc::htOdbc3xManager.clearQueues();

htResultTriple = uesqlc::htOdbc3xManager.execute ("DROP TABLE WORKERS CASCADE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
uesqlc::htOdbc3xManager.clearResult (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not ODBC 3.x
#line 451 "../test-000-cursors.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
