#include <mpcl/util/logging/log.hh>
#include "odbc-3.x.hh"

#line 1 "../test-003-iterators.uesql.cc"
/*
*  Name:         test-003-iterators.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Test for SQL iterators
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 1999-2001  Francisco Vides Fernandez <pax@dedalo-ing.com>
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/exceptions.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>
#include <uesqlc/data_source.hh>
#include <uesqlc/iterator.hh>
#include <uesqlc/query.hh>

using uesqlc::ComposeSchemaLocation;
using uesqlc::ComposeUserLogin;
using uesqlc::IType;
using uesqlc::TDate;
using uesqlc::TIdentifier;
using uesqlc::TInteger;
using uesqlc::TIterator;
using uesqlc::TQuery;
using uesqlc::TReal;
using uesqlc::TSqlerrorException;
using uesqlc::TString;


//
//  L O C A L   C L A S S E S
//

class TTestInstance
{

  public:

    TDate      tAttrib0;
    TInteger   tAttrib1;
    TReal      tAttrib2;
    TString    tAttrib3;

    
  public:

    TTestInstance() :
      tAttrib0()    ,
      tAttrib1()    ,
      tAttrib2()    ,
      tAttrib3()    {}


  public: 

    IType& operator [] (unsigned int uiI)
    {
      switch (uiI)
      {
        case 0:
        {
          return tAttrib0;
          break;
        }
        case 1:
        {
          return tAttrib1;
          break;
        }
        case 2:
        {
          return tAttrib2;
          break;
        }
        case 3:
        {
          return tAttrib3;
          break;
        }
        default:        
        {
          throw mpcl::TConstraintException ("index out of range", __FILE__, __LINE__);
        }
      }
      return tAttrib0; // never reached
    }

    bool operator == (const TTestInstance& rktINSTANCE) const
    {
      return 
        ( ( tAttrib0 == rktINSTANCE.tAttrib0 ) &&
          ( tAttrib1 == rktINSTANCE.tAttrib1 ) &&
          ( tAttrib2 == rktINSTANCE.tAttrib2 ) &&
          ( tAttrib3 == rktINSTANCE.tAttrib3 )  );
    }

};  // class TTestInstance


template <typename TChar, typename TTraits>
std::basic_ostream<TChar, TTraits>&
operator << (std::basic_ostream<TChar, TTraits>& rtOSTREAM, TTestInstance& rtINSTANCE)
{

  for (register unsigned int I = 0; ( I < 4 ) ;++I)
  {
    rtOSTREAM << rtINSTANCE [I].get() << std::endl;
  }
  return rtOSTREAM;

}  // operator <<()


///  SQL back-end for \a TQuery.
class TDataSource : public uesqlc::IDataSource<TTestInstance>
{

  protected:

    //
    //  S E L E C T O R S
    //

    void closeCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
#line 148 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 149 "../test-003-iterators.uesql.cc"

#line 150 "../test-003-iterators.uesql.cc"
{
mpcl::text::TString   hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TIdentifier (rkyCURSOR_NAME));

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("$z");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

uesqlc::htOdbc3xManager.clearResult (hrtResultTriple);
uesqlc::htOdbc3xManager.removeCursor (hyCursorName);
}
}
}  // not ODBC 3.x
#line 151 "../test-003-iterators.uesql.cc"

    }

    size_type declareCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
      TInteger   iAux;

#line 159 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 160 "../test-003-iterators.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
int                              hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnOutputQueue (&iAux);

htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT COUNT (*) FROM INSTANCE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  
throw uesqlc::TNotFoundException (__FILE__, __LINE__);
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
  bool   gIsSingleRow;
  int    hiLength;

SQLColAttribute (htResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( htResultTriple.tSqlhstmt                                                                ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
  gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_SUCCESS );
  if ( gIsSingleRow )
  {
    gIsSingleRow = ( SQLFetch (htResultTriple.tSqlhstmt) == SQL_NO_DATA );
  }
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  if ( gIsSingleRow )
  {
    uesqlc::htOdbc3xManager.updateOutputVariables();
  }
  else
  {
    // 
    //  Condition:  cardinality violation
    // 
    uesqlc::SQLCODE    = -1;
    uesqlc::SQLMESSAGE = "";
    uesqlc::SQLSTATE   = "21000";
    throw TSqlerrorException (__FILE__, __LINE__);
  }
}
}
#line 162 "../test-003-iterators.uesql.cc"
{
uesqlc::odbc_3_x::TResultTriple   htResultTriple;
mpcl::text::TString              hyCursorName;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TIdentifier (rkyCURSOR_NAME));

//
//  First, it instantiates <cursor name> to follow the input order.
//
hyCursorName   = uesqlc::htOdbc3xManager.instantiate ("$z");
htResultTriple = uesqlc::htOdbc3xManager.execute ("SELECT ATTRIB0, ATTRIB1, ATTRIB2, ATTRIB3 FROM INSTANCE;");
uesqlc::htOdbc3xManager.diagnose (htResultTriple);
if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  uesqlc::htOdbc3xManager.clearResult (htResultTriple);
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
  SQLSetCursorName (htResultTriple.tSqlhstmt, (SQLCHAR*) hyCursorName.c_str(), SQL_NTS);
  uesqlc::htOdbc3xManager.addCursor (hyCursorName, htResultTriple);
}
}
}  // not ODBC 3.x
#line 163 "../test-003-iterators.uesql.cc"

      return (size_type) iAux;
    }

    void fetchAbsolute ( const mpcl::text::TString& rkyCURSOR_NAME ,
                         TTestInstance&             rtINSTANCE     ,
                         size_type                  zINDEX         )
    {
      int   iIndex = zINDEX;

#line 174 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 180 "../test-003-iterators.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TInteger (iIndex));
uesqlc::htOdbc3xManager.pushOnInputQueue (TIdentifier (rkyCURSOR_NAME));
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib1);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib2);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib3);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("$z");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetchScroll (hrtResultTriple.tSqlhstmt, SQL_FETCH_ABSOLUTE, std::atoi (uesqlc::htOdbc3xManager.instantiate ("$z").c_str()));
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  
throw uesqlc::TNotFoundException (__FILE__, __LINE__);
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 181 "../test-003-iterators.uesql.cc"

    }

    void fetchNext ( const mpcl::text::TString& rkyCURSOR_NAME ,
                     TTestInstance&             rtINSTANCE     )
    {
#line 188 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 193 "../test-003-iterators.uesql.cc"
{
mpcl::text::TString   hyCursorName;
int                   hI = 0;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TIdentifier (rkyCURSOR_NAME));
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib0);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib1);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib2);
uesqlc::htOdbc3xManager.pushOnOutputQueue (&rtINSTANCE.tAttrib3);

hyCursorName = uesqlc::htOdbc3xManager.instantiate ("$z");
if ( !uesqlc::htOdbc3xManager.hasCursor (hyCursorName) )
{
  //
  //  Condition:  invalid cursor name
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "34000";
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
int                                   hiLength;
uesqlc::odbc_3_x::TResultTriple&   hrtResultTriple (uesqlc::htOdbc3xManager.getCursor (hyCursorName));

if ( !hrtResultTriple.gIsBound )
{
hrtResultTriple.gIsBound = true;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
SQLColAttribute (hrtResultTriple.tSqlhstmt, hI + 1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &hiLength);
SQLBindCol ( hrtResultTriple.tSqlhstmt                                                               ,
             hI + 1                                                                                  ,
             SQL_C_CHAR                                                                              ,
             (SQLPOINTER) uesqlc::htOdbc3xManager.getFromOutputQueue (hI)->getBuffer (hiLength + 1) ,
             hiLength + 1                                                                            ,
             uesqlc::htOdbc3xManager.getResultLengthFromOutputQueue (hI)                            );
++hI;
}
hrtResultTriple.tSqlreturn = SQLFetch (hrtResultTriple.tSqlhstmt);
uesqlc::htOdbc3xManager.diagnose (hrtResultTriple);
if ( uesqlc::htOdbc3xManager.isNotFoundCondition() )
{
  
throw uesqlc::TNotFoundException (__FILE__, __LINE__);
}
else if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
{
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
  uesqlc::htOdbc3xManager.updateOutputVariables();
}
}
}
}  // not ODBC 3.x
#line 194 "../test-003-iterators.uesql.cc"

    }

    void openCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
#line 200 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 201 "../test-003-iterators.uesql.cc"

#line 202 "../test-003-iterators.uesql.cc"
uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (TIdentifier (rkyCURSOR_NAME));


}  // not ODBC 3.x
#line 203 "../test-003-iterators.uesql.cc"

    }

    void setErrorException (void)
    {
#line 209 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 210 "../test-003-iterators.uesql.cc"

}  // not ODBC 3.x
#line 211 "../test-003-iterators.uesql.cc"

    }

};  // class TDataSource


typedef uesqlc::TQuery<TTestInstance, TDataSource>  TTestQuery;


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _TestIteratorOperators (void)
{

  // Test ++I

  // Test I++

}   // _TestOperators()


int main (int argc, const char** argv)
{

  mpcl::util::prefs::TConfigProcessor   tConfigProcessor (argc, argv);
  TString                               yDbmsTestSchemaLocation;
  TString                               yDbmsUserLogin;
  const char*                           pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  TEST_INIT ("Test for Cursor inside C++ Iterators");
  TEST_START_WATCH;
  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::util::logging::tLog.also (std::cerr);
    }
    yDbmsTestSchemaLocation = ComposeSchemaLocation ( tConfigProcessor ["dbms-server-name"].getValue() ,
                                                      "TEST_ITERATORS_SCHEMA"                          );
    yDbmsUserLogin          = ComposeUserLogin ("test", "test");
    TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
    TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
#line 295 "../test-003-iterators.uesql.cc"
{
//
//  ODBC 3.x instructions.
//
#line 299 "../test-003-iterators.uesql.cc"

#line 300 "../test-003-iterators.uesql.cc"
{
mpcl::text::TString              hySqlSchemaLocation;
mpcl::text::TString              hyDatabaseName;
mpcl::text::TString              hyConnectionName;
mpcl::text::TString              hyUserLogin;
mpcl::text::TString::size_type   zSolidusPos;

uesqlc::htOdbc3xManager.clearQueues();
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsTestSchemaLocation);
uesqlc::htOdbc3xManager.pushOnInputQueue (yDbmsUserLogin);

//
//  '$z' has been omitted.
//
hySqlSchemaLocation = uesqlc::htOdbc3xManager.instantiate ("$z");
hyConnectionName    = "DEFAULT";
hyUserLogin         = uesqlc::htOdbc3xManager.instantiate ("$z");
hySqlSchemaLocation.suppress ('\'');
zSolidusPos = hySqlSchemaLocation.find ('/');
if ( zSolidusPos == mpcl::text::TString::npos )
{
  hyDatabaseName = hySqlSchemaLocation;
}
else
{
  hyDatabaseName = hySqlSchemaLocation.substr (0, zSolidusPos);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  uesqlc::htOdbc3xManager.unbindConnection (hyConnectionName);
}
if ( uesqlc::htOdbc3xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  throw TSqlerrorException (__FILE__, __LINE__);
}
else
{
  uesqlc::htOdbc3xManager.connect ( hyConnectionName ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htOdbc3xManager.isSqlerrorCondition() )
  {
    throw TSqlerrorException (__FILE__, __LINE__);
  }
  else
  {
    uesqlc::htOdbc3xManager.setCurrentConnection (hyConnectionName);
  }
}
}
}  // not ODBC 3.x
#line 301 "../test-003-iterators.uesql.cc"

    // <TODO comment="this will be the tests to run">
    _TestIteratorOperators();
    // </TODO>    
    
    {
      //
      //  Iterate using a query.
      //
      TTestQuery             tTestQuery;
      TTestQuery::iterator   tTestIterator = tTestQuery.begin();

      //
      //  This one can test if an iterator's constructor returns a
      //  valid one, being valid the data set.
      //
      if ( tTestIterator == tTestQuery.end() )
      {
        TEST_FAIL;
      }

      //
      //  Iterating over all registers.
      //
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2000, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         0);
      TEST_NUMBERS (tTestIterator->tAttrib2,         0.0);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 0");
      ++tTestIterator;
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2001, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         1);
      TEST_NUMBERS (tTestIterator->tAttrib2,         1.1);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 1");
      ++tTestIterator;
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2002, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         2);
      TEST_NUMBERS (tTestIterator->tAttrib2,         2.2);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 2");

      //
      //  Iterating through the container as usual.
      //
      tTestIterator = tTestQuery.begin();
      for (; ( tTestIterator != tTestQuery.end() ) ;++tTestIterator)
      {
        if ( !TEST_IN_SILENT_MODE )
        {
          std::cout << *tTestIterator;
        }
      }
    }
  }
  catch (const uesqlc::TSqlerrorException& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesqlc::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
  }
  catch (const mpcl::TConstraintException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (const mpcl::TException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
