#include <mpcl/util/logging/log.hh>
#include "postgresql-libpq-6.2.x.hh"

#line 1 "../test-000-cursors.uesql.cc"
/*
*  Name:         test-000-cursors.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 1996-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using std::cerr;
  using std::endl;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TString       yVar0;
  TString       yVar1;
  TReal         tHours;
  TInteger      tZero (0L);
  TIdentifier   yAttribName ("work_code");
  bool          gEmpty       = true;
  int           iRecordCount = 0;

  TEST_INIT ("test for cursors");

#line 84 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 85 "../test-000-cursors.uesql.cc"

#line 86 "../test-000-cursors.uesql.cc"

#line 90 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT NAME, HOURS FROM WORKERS WHERE ( NAME = 'nobody' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 91 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();


}  // not PostgreSQL LIBPQ 6.2.x
#line 92 "../test-000-cursors.uesql.cc"

  while ( true )
  {
#line 96 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 97 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar1);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 98 "../test-000-cursors.uesql.cc"

    gEmpty = false;
  }

loop_end:
#line 104 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 105 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesqlc::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 106 "../test-000-cursors.uesql.cc"

  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                       HOURS\n");
    fprintf (hptTestTargetFile, "========= ========================================== =====\n");
  }
#line 122 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 126 "../test-000-cursors.uesql.cc"

#line 127 "../test-000-cursors.uesql.cc"

#line 132 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yAttribName);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT $z, NAME, HOURS FROM WORKERS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 133 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();


#line 134 "../test-000-cursors.uesql.cc"
{
PGresult*             ptPgresult;
mpcl::text::TString   hySqlInstructionAux;
mpcl::text::TString   hyCursorName = "MYCURSOR";
int                   hI           = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (tZero);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tHours);

{
PGresult*   ptPgresult;

ptPgresult = uesqlc::htPostgresql62xManager.execute (mpcl::text::Format ("MOVE BACKWARD ALL IN %s;", hyCursorName.c_str()));
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
if ( 0 == std::atoi (uesqlc::htPostgresql62xManager.instantiate ("$z").c_str()) )
{
  //
  //  Condition:  no data
  // 
  uesqlc::SQLCODE    = 101;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "02000";
  goto loop_end_2;
}
else
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute (mpcl::text::Format ("MOVE FORWARD $z IN %s;", hyCursorName.c_str()));
  uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
}
}
ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end_2;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 135 "../test-000-cursors.uesql.cc"

  TEST_FAIL;
  goto next;

loop_end_2:
  TEST_PASS;

next:  
  while ( true )
  {
#line 146 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 147 "../test-000-cursors.uesql.cc"

#line 148 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tHours);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end_3;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 149 "../test-000-cursors.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5.1f\n", yVar0.get().c_str(), yVar1.get().c_str(), (double) tHours);
    }
    ++iRecordCount;
  }

loop_end_3:
#line 159 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 163 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesqlc::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 164 "../test-000-cursors.uesql.cc"

  if ( iRecordCount > 0 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  TString*           pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
#line 262 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 266 "../test-000-cursors.uesql.cc"

#line 267 "../test-000-cursors.uesql.cc"

#line 272 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = "DEFAULT";
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
  if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
  {
    uesqlc::htPostgresql62xManager.unbindConnection (hyConnectionName);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 281 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE JOBS (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 288 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE TABLE WORKERS (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 293 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("ALTER TABLE WORKERS ADD COLUMN STATE CHAR (40);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 294 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INT4;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 299 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yCodeValue);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DELETE FROM JOBS WHERE ( CODE = $z );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 300 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DELETE FROM JOBS WHERE ( CODE = '001' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 301 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DELETE FROM WORKERS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 306 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO JOBS VALUES ('001', 100);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 307 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO JOBS VALUES ('002', 250);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 308 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 309 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 310 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 311 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 312 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 313 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'william', NULL, 'Missouri', 5550005);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 314 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'peter', NULL, 'Arkansas', 5550006);");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 315 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 316 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 321 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tCondition);

uesqlc::htPostgresql62xManager.popFromOutputQueue()->set (uesqlc::htPostgresql62xManager.instantiate ("( WORK_CODE = CODE )"));
#line 326 "../test-000-cursors.uesql.cc"

#line 333 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesqlc::htPostgresql62xManager.pushOnInputQueue (tCondition);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT HOURS FROM WORKERS, JOBS WHERE $z;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto cardinality_check;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 334 "../test-000-cursors.uesql.cc"

    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::cardinality_violation::pkcNO_SUBCLASS);
#line 340 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 344 "../test-000-cursors.uesql.cc"

#line 349 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tCount);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( HOURS IS NULL );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 354 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT WORK_CODE, HOURS FROM WORKERS WHERE ( HOURS IS NULL );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 355 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();


#line 356 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tHours);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto not_found;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ", " << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
#line 357 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesqlc::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 358 "../test-000-cursors.uesql.cc"

    TEST_NUMBERS (tCount,          4);
    TEST_NUMBERS (tCount.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),  false);
    TEST_NUMBERS (tHours.isNull(), true);
    
    yCursorName = "mycursor";
#line 366 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 370 "../test-000-cursors.uesql.cc"

#line 375 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tCount);

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  uesqlc::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesqlc::htPostgresql62xManager.end();
}
}
#line 380 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tCondition);

uesqlc::htPostgresql62xManager.popFromOutputQueue()->set (uesqlc::htPostgresql62xManager.instantiate ("( WORKERS.NAME = 'myname' )"));
#line 381 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&tCondition);
uesqlc::htPostgresql62xManager.pushOnInputQueue (tCondition);

uesqlc::htPostgresql62xManager.popFromOutputQueue()->set (uesqlc::htPostgresql62xManager.instantiate ("( $z AND ( WORKERS.NAME = NAME ) )"));
#line 388 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (tCondition);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE $z;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 389 "../test-000-cursors.uesql.cc"

    TEST_FUNCTION (_TestCursors());
#line 392 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 396 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.begin();
ptPgresult = uesqlc::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 397 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();


#line 398 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yCursorName);
uesqlc::htPostgresql62xManager.pushOnOutputQueue (&yVar0);

ptPgresult = uesqlc::htPostgresql62xManager.execute ("FETCH IN $z;");
uesqlc::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isNotFoundCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto not_found;
}
else if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << mpcl::util::logging::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << uesqlc::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::util::logging::tLog << ");\n";
#endif  // not _UESQL_DEBUG__
  uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
}
}
#line 399 "../test-000-cursors.uesql.cc"
uesqlc::htPostgresql62xManager.clearQueues();

uesqlc::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesqlc::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 400 "../test-000-cursors.uesql.cc"

    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::successful_completion::pkcNO_SUBCLASS);
    TEST_NUMBERS (tCount,        4);
    TEST_STRINGS (yVar0.c_str(), "001");
#line 405 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 409 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("REVOKE ALL ON JOBS FROM PUBLIC;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 410 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'RESTRICT' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("REVOKE ALL ON WORKERS FROM PUBLIC;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 415 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("GRANT SELECT ON JOBS TO PUBLIC;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 416 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("GRANT SELECT ON WORKERS TO PUBLIC;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 417 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("GRANT DELETE ON WORKERS TO PUBLIC;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 422 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 427 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DROP VIEW VIEW1;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 432 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DROP TABLE JOBS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 437 "../test-000-cursors.uesql.cc"
{
uesqlc::htPostgresql62xManager.clearQueues();

if ( uesqlc::htPostgresql62xManager.existsDefaultConnection() )
{
  uesqlc::htPostgresql62xManager.resetStatus();
  uesqlc::htPostgresql62xManager.unbindCurrentConnection();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08003";
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 438 "../test-000-cursors.uesql.cc"


#line 441 "../test-000-cursors.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 445 "../test-000-cursors.uesql.cc"
{
mpcl::text::TString   hySqlServerDescriptor;
mpcl::text::TString   hySqlServerName;
mpcl::text::TString   hyDatabaseName;
mpcl::text::TString   hyConnectionName;
mpcl::text::TString   hyUserLogin;

uesqlc::htPostgresql62xManager.clearQueues();
uesqlc::htPostgresql62xManager.pushOnInputQueue (yDbmsSchemaLocation);
uesqlc::htPostgresql62xManager.pushOnInputQueue (*pyDbmsUserLogin);

{
  mpcl::text::TString::size_type   hzBarIndex;

  hySqlServerDescriptor = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hyConnectionName      = uesqlc::htPostgresql62xManager.instantiate ("_'myconnection'");
  hyUserLogin           = uesqlc::htPostgresql62xManager.instantiate ("$z");
  hySqlServerDescriptor.suppress ('\'');
  hyConnectionName.suppress ('\'');
  hzBarIndex = hySqlServerDescriptor.find ('/');
  if ( hzBarIndex == mpcl::text::TString::npos )
  {
    hySqlServerName = hySqlServerDescriptor;
  }
  else
  {
    hySqlServerName = hySqlServerDescriptor.substr (0, hzBarIndex);
    hyDatabaseName  = hySqlServerDescriptor.substr (hzBarIndex + 1);
  }
}
if ( uesqlc::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesqlc::SQLCODE    = -1;
  uesqlc::SQLMESSAGE = "";
  uesqlc::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesqlc::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesqlc::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 450 "../test-000-cursors.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesqlc::htPostgresql62xManager.clearQueues();

ptPgresult = uesqlc::htPostgresql62xManager.execute ("DROP TABLE WORKERS;");
uesqlc::htPostgresql62xManager.diagnose (ptPgresult);
uesqlc::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesqlc::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 451 "../test-000-cursors.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
