/*
*  Name:         test-000-cursors.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 1996-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using std::cerr;
  using std::endl;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TString       yVar0;
  TString       yVar1;
  TReal         tHours;
  TInteger      tZero (0L);
  TIdentifier   yAttribName ("work_code");
  bool          gEmpty       = true;
  int           iRecordCount = 0;

  TEST_INIT ("test for cursors");

  extern "UESQL::default"
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    declare mycursor cursor for
      select name, hours
        from workers
        where ( name = 'nobody' );
    open mycursor;
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from mycursor into :yVar0, :yVar1;
    }
    gEmpty = false;
  }

loop_end:
  extern "UESQL::default" 
  {
    close mycursor;
  }
  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                       HOURS\n");
    fprintf (hptTestTargetFile, "========= ========================================== =====\n");
  }
  extern "UESQL::default"
  {
    --
    --  WHENEVER
    --
    whenever sqlerror  goto error;
    whenever not found goto loop_end_2;

    --
    --  DECLARE CURSOR/OPEN/FETCH
    --
    declare mycursor cursor for select :yAttribName, name, hours from workers;
    open mycursor;
    fetch absolute :tZero from mycursor into :yVar0, :yVar1, :tHours;
  }
  TEST_FAIL;
  goto next;

loop_end_2:
  TEST_PASS;

next:  
  while ( true )
  {
    extern "UESQL::default"
    {
      whenever not found goto loop_end_3;
      fetch from mycursor into :yVar0, :yVar1, :tHours;
    }
    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5.1f\n", yVar0.get().c_str(), yVar1.get().c_str(), (double) tHours);
    }
    ++iRecordCount;
  }

loop_end_3:
  extern "UESQL::default"
  {
    --
    --  CLOSE CURSOR
    --
    close mycursor;
  }
  if ( iRecordCount > 0 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the schema  location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeSchemaLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsUserLogin;
  TString            yDbmsSchemaLocation;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  TString*           pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin      = ComposeUserLogin      ("test",          "test");
    yDbmsSchemaLocation = ComposeSchemaLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror  goto error;
      whenever not found goto not_found;
      
      --
      --  CONNECT TO
      --
      connect to :yDbmsSchemaLocation user :yDbmsUserLogin;

      --
      --  CREATE TABLE
      --
      create table jobs
      (
        code         character (3) not null ,
        worker_count integer
      );

      create table workers
      (
        work_code character (3)  not null ,
        name      character (40) not null ,
        hours     decimal (5)
      );

      --
      --  ALTER TABLE
      --
      alter table workers add column state character (40);
      alter table workers add column phone_number integer;

      --
      --  DELETE
      --
      delete from jobs where ( code = :yCodeValue );
      delete from jobs where ( code = '001' );
      delete from workers;

      --
      --  INSERT
      --
      insert into jobs values ('001', 100);
      insert into jobs values ('002', 250);
      insert into workers values ('001', 'myname',   40,   'Andalucia',   5550000);
      insert into workers values ('001', 'yourname', 35,   'Extremadura', 5550001);
      insert into workers values ('001', 'hisname',  25,   'Catalunya',   5550002);
      insert into workers values ('002', 'john',     35,   'New England', 5550003);
      insert into workers values ('002', 'james',    35,   'Washington',  5550004);
      insert into workers values ('002', 'william',  NULL, 'Missouri',    5550005);
      insert into workers values ('002', 'peter',    NULL, 'Arkansas',    5550006);
      insert into workers select work_code, name, hours, state, phone_number from workers where ( name = 'myname' );
      insert into workers select work_code, name, hours, state, phone_number from workers;

      --
      --  DECLARE CONDITION
      --
      declare :tCondition condition where ( work_code = code );

      --
      --  WHENEVER
      --
      whenever sqlerror goto cardinality_check;

      --
      --  SELECT for a single row.
      --
      --    Cardinality violation, so ( SQLSTATE == '21000' ).
      --
      select hours into :yVar1 from workers, jobs where :tCondition;
    }
    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::cardinality_violation::pkcNO_SUBCLASS);
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :tCount from workers where ( hours is null );

      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare mycursor cursor for select work_code, hours from workers where ( hours is null );
      open mycursor;
      fetch from mycursor into :yVar0, :tHours;
      close mycursor;
    }
    TEST_NUMBERS (tCount,          4);
    TEST_NUMBERS (tCount.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),  false);
    TEST_NUMBERS (tHours.isNull(), true);
    
    yCursorName = "mycursor";
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :tCount from workers where name = 'myname';

      --
      --  DECLARE CONDITION
      --
      declare :tCondition condition where ( workers.name = 'myname' );
      declare :tCondition condition where ( :tCondition and ( workers.name = name ) );

      --
      --  UPDATE
      --
      update workers
        set phone_number = workers.phone_number
        where :tCondition;
    }
    TEST_FUNCTION (_TestCursors());
    extern "UESQL::default" 
    {
      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare mycursor cursor for select code from jobs where code = '001';
      open mycursor;
      fetch from :yCursorName into :yVar0;
      close mycursor;
    }
    TEST_STRINGS (uesqlc::SQLSTATE.c_str(), uesqlc::successful_completion::pkcNO_SUBCLASS);
    TEST_NUMBERS (tCount,        4);
    TEST_STRINGS (yVar0.c_str(), "001");
    extern "UESQL::default" 
    {
      --
      --  REVOKE
      --
      revoke all privileges on jobs    from public cascade;
      revoke all privileges on workers from public restrict;

      --
      --  GRANT
      --
      grant select on jobs    to public;
      grant select on workers to public;
      grant delete on workers to public;

      --
      --  CREATE VIEW
      --
      create view view1 as select * from jobs;

      --
      --  DROP VIEW
      --
      drop view view1 cascade;

      --
      --  DROP TABLE
      --
      drop table jobs cascade;

      --
      --  DISCONNECT
      --
      disconnect current;
    }

    extern "UESQL::default"
    {
      --
      --  CONNECT TO
      --
      connect to :yDbmsSchemaLocation as 'myconnection' user :*pyDbmsUserLogin;

      --
      --  DROP TABLE
      --
      drop table workers cascade;
    }
    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
