/*
*  Name:         test-003-iterators.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Test for SQL iterators
*  Date:         $Date: 2003/10/06 13:21:36 $
*  Revision:     $Revision: 1.3 $
*
*  Copyright (C) 1999-2001  Francisco Vides Fernandez <pax@dedalo-ing.com>
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/exceptions.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>
#include <uesqlc/data_source.hh>
#include <uesqlc/iterator.hh>
#include <uesqlc/query.hh>

using uesqlc::ComposeSchemaLocation;
using uesqlc::ComposeUserLogin;
using uesqlc::IType;
using uesqlc::TDate;
using uesqlc::TIdentifier;
using uesqlc::TInteger;
using uesqlc::TIterator;
using uesqlc::TQuery;
using uesqlc::TReal;
using uesqlc::TSqlerrorException;
using uesqlc::TString;


//
//  L O C A L   C L A S S E S
//

class TTestInstance
{

  public:

    TDate      tAttrib0;
    TInteger   tAttrib1;
    TReal      tAttrib2;
    TString    tAttrib3;

    
  public:

    TTestInstance() :
      tAttrib0()    ,
      tAttrib1()    ,
      tAttrib2()    ,
      tAttrib3()    {}


  public: 

    IType& operator [] (unsigned int uiI)
    {
      switch (uiI)
      {
        case 0:
        {
          return tAttrib0;
          break;
        }
        case 1:
        {
          return tAttrib1;
          break;
        }
        case 2:
        {
          return tAttrib2;
          break;
        }
        case 3:
        {
          return tAttrib3;
          break;
        }
        default:        
        {
          throw mpcl::TConstraintException ("index out of range", __FILE__, __LINE__);
        }
      }
      return tAttrib0; // never reached
    }

    bool operator == (const TTestInstance& rktINSTANCE) const
    {
      return 
        ( ( tAttrib0 == rktINSTANCE.tAttrib0 ) &&
          ( tAttrib1 == rktINSTANCE.tAttrib1 ) &&
          ( tAttrib2 == rktINSTANCE.tAttrib2 ) &&
          ( tAttrib3 == rktINSTANCE.tAttrib3 )  );
    }

};  // class TTestInstance


template <typename TChar, typename TTraits>
std::basic_ostream<TChar, TTraits>&
operator << (std::basic_ostream<TChar, TTraits>& rtOSTREAM, TTestInstance& rtINSTANCE)
{

  for (register unsigned int I = 0; ( I < 4 ) ;++I)
  {
    rtOSTREAM << rtINSTANCE [I].get() << std::endl;
  }
  return rtOSTREAM;

}  // operator <<()


///  SQL back-end for \a TQuery.
class TDataSource : public uesqlc::IDataSource<TTestInstance>
{

  protected:

    //
    //  S E L E C T O R S
    //

    void closeCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TSqlerrorException;
        close :TIdentifier (rkyCURSOR_NAME);
      }
    }

    size_type declareCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
      TInteger   iAux;

      extern "UESQL::default"
      {
        select count (*) into :iAux from instance;
        declare :TIdentifier (rkyCURSOR_NAME) cursor for 
          select attrib0, attrib1, attrib2, attrib3 from instance;
      }
      return (size_type) iAux;
    }

    void fetchAbsolute ( const mpcl::text::TString& rkyCURSOR_NAME ,
                         TTestInstance&             rtINSTANCE     ,
                         size_type                  zINDEX         )
    {
      int   iIndex = zINDEX;

      extern "UESQL::default"
      {
        fetch absolute :TInteger (iIndex)
          from :TIdentifier (rkyCURSOR_NAME)
          into :rtINSTANCE.tAttrib0 ,
               :rtINSTANCE.tAttrib1 ,
               :rtINSTANCE.tAttrib2 ,
               :rtINSTANCE.tAttrib3 ;
      }
    }

    void fetchNext ( const mpcl::text::TString& rkyCURSOR_NAME ,
                     TTestInstance&             rtINSTANCE     )
    {
      extern "UESQL::default"
      {
        fetch from :TIdentifier (rkyCURSOR_NAME)
          into :rtINSTANCE.tAttrib0 ,
               :rtINSTANCE.tAttrib1 ,
               :rtINSTANCE.tAttrib2 ,
               :rtINSTANCE.tAttrib3 ;
      }
    }

    void openCursor (const mpcl::text::TString& rkyCURSOR_NAME)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TSqlerrorException;
        open :TIdentifier (rkyCURSOR_NAME);
      }
    }

    void setErrorException (void)
    {
      extern "UESQL::default"
      {
        whenever sqlerror throw TSqlerrorException;
      }
    }

};  // class TDataSource


typedef uesqlc::TQuery<TTestInstance, TDataSource>  TTestQuery;


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _TestIteratorOperators (void)
{

  // Test ++I

  // Test I++

}   // _TestOperators()


int main (int argc, const char** argv)
{

  mpcl::util::prefs::TConfigProcessor   tConfigProcessor (argc, argv);
  TString                               yDbmsTestSchemaLocation;
  TString                               yDbmsUserLogin;
  const char*                           pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  TEST_INIT ("Test for Cursor inside C++ Iterators");
  TEST_START_WATCH;
  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::util::logging::tLog.also (std::cerr);
    }
    yDbmsTestSchemaLocation = ComposeSchemaLocation ( tConfigProcessor ["dbms-server-name"].getValue() ,
                                                      "TEST_ITERATORS_SCHEMA"                          );
    yDbmsUserLogin          = ComposeUserLogin ("test", "test");
    TEST_WRITE_INFO ("DBMS-TEST-SCHEMA-LOCATION = %s", yDbmsTestSchemaLocation.c_str());
    TEST_WRITE_INFO ("DBMS-USER-LOGIN           = %s", yDbmsUserLogin.c_str());
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition throw an exception
      --
      whenever sqlerror throw TSqlerrorException;
      connect to :yDbmsTestSchemaLocation user :yDbmsUserLogin;
    }
    // <TODO comment="this will be the tests to run">
    _TestIteratorOperators();
    // </TODO>    
    
    {
      //
      //  Iterate using a query.
      //
      TTestQuery             tTestQuery;
      TTestQuery::iterator   tTestIterator = tTestQuery.begin();

      //
      //  This one can test if an iterator's constructor returns a
      //  valid one, being valid the data set.
      //
      if ( tTestIterator == tTestQuery.end() )
      {
        TEST_FAIL;
      }

      //
      //  Iterating over all registers.
      //
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2000, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         0);
      TEST_NUMBERS (tTestIterator->tAttrib2,         0.0);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 0");
      ++tTestIterator;
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2001, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         1);
      TEST_NUMBERS (tTestIterator->tAttrib2,         1.1);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 1");
      ++tTestIterator;
      TEST_NUMBERS (tTestIterator->tAttrib0,         TDate (2002, 12, 20));
      TEST_NUMBERS (tTestIterator->tAttrib1,         2);
      TEST_NUMBERS (tTestIterator->tAttrib2,         2.2);
      TEST_STRINGS (tTestIterator->tAttrib3.c_str(), "valor 2");

      //
      //  Iterating through the container as usual.
      //
      tTestIterator = tTestQuery.begin();
      for (; ( tTestIterator != tTestQuery.end() ) ;++tTestIterator)
      {
        if ( !TEST_IN_SILENT_MODE )
        {
          std::cout << *tTestIterator;
        }
      }
    }
  }
  catch (const uesqlc::TSqlerrorException& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesqlc::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
  }
  catch (const mpcl::TConstraintException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (const mpcl::TException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
