/*
*  Name:      TConnectionData.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Class for connection data.
*  Date:      $Date: 2003/06/24 23:29:04 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.util.Hashtable;


///  Connection data class.
public class TConnectionData
{

  ///  Connection.
  private Connection   tConnection;

  ///  Map that binds cursor names to result sets.
  private Hashtable   tResultSetHashtable;

  ///  Host name.
  public String   yHostName;

  ///  Schema name.
  public String   ySchemaName;

  ///  User name.
  public String   yUserName;

  ///  User password.
  public String   yUserPassword;


  //
  //  C O N S T R U C T O R S
  //

  public TConnectionData (Connection tCONNECTION)
  {
    tConnection         = tCONNECTION;
    tResultSetHashtable = new Hashtable();
  }

  /**
  *  Binds in the cursor map a cursor named \a kyCURSOR_NAME to \a tRESULT_SET.
  *  @param kyCURSOR_NAME Cursor name.
  *  @param tRESULT_SET   Result set instance.
  */
  public void bind (final String kyCURSOR_NAME, ResultSet tRESULT_SET)
  {
    tResultSetHashtable.put (kyCURSOR_NAME, tRESULT_SET);
  }

  /**
  *  Unbinds the cursor named \a kyCURSOR_NAME from the cursor map.
  *  @param kyCURSOR_NAME Cursor name.
  */
  public void unbind (final String kyCURSOR_NAME)
  {
    tResultSetHashtable.remove (kyCURSOR_NAME);
  }


  //
  //  S E L E C T O R S
  //

  public Connection getConnection()
  {
    return tConnection;
  }

  /**
  *  Gets the result set bound to the cursor name \a kyCURSOR_NAME.
  *  @param kyCURSOR_NAME Cursor name.
  *  @return The result set bound to the cursor.
  */
  public ResultSet getCursor (final String kyCURSOR_NAME)
  {
    return (ResultSet) tResultSetHashtable.get (kyCURSOR_NAME);
  }

}  // class TConnectionData
