/*
*  Name:      TDate.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type date.
*  Date:      $Date: 2003/07/10 00:42:28 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.sql.Date;
import java.util.GregorianCalendar;


///  SQL type for dates.
public class TDate implements IType
{

  private Date   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TDate() {}

  /**
  *  Builds a new instance.
  *  @param ktDATE Source date.
  *  @post ( isNull() == ( null == ktDATE ) )
  */
  public TDate (final Date ktDATE)
  {
    if ( ktDATE != null )
    {
      tValue = (Date) ktDATE.clone();
    }
  }

  /**
  *  Builds a new instance.
  *  @param kiYEAR  Source year.
  *  @param kiMONTH Source month (1-12).
  *  @param kiDAY   Source day (1-31).
  *  @post ( !isNull() )
  */
  public TDate (final int kiYEAR, final int kiMONTH, final int kiDAY)
  {
    long   lTimeInMillis;

    lTimeInMillis = new GregorianCalendar (kiYEAR, kiMONTH - 1, kiDAY).getTimeInMillis();
    tValue        = new Date (lTimeInMillis);
  }

  /**
  *  Builds a new instance.
  *  @param kyDATE Source date.
  *  @post ( isNull() == ( null == kyDATE ) )
  */
  public TDate (final String kyDATE)
  {
    if ( kyDATE != null )
    {
      tValue = Date.valueOf (kyDATE);
    }
  }

  public IType set (final Object ktVALUE)
  {
    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof Date )
    {
      tValue = (Date) ((Date) ktVALUE).clone();
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;
  }


  //
  //  S E L E C T O R S
  //

  public String get()
  {

    String   yResult;

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = "'" + tValue + "'";
    }
    return yResult;

  }  // get()

  public Date dateValue()
  {
    return tValue;
  }

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public String toString()
  {
    return ( tValue == null ) ? null : tValue.toString();
  }

}  // class TDate
